package com.xxrv.video.activity;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.frame.rv.config.RvFrameConfig;
import com.ruiwenliu.wrapper.base.BaseBean;
import com.ruiwenliu.wrapper.base.BaseStatusActivity;
import com.ruiwenliu.wrapper.util.glide.GlideApp;
import com.ruiwenliu.wrapper.weight.TitleView;
import com.xxrv.video.R;
import com.xxrv.video.R2;
import com.xxrv.video.api.VideoApi;
import com.xxrv.video.audio_x.VoiceManager;
import com.xxrv.video.base.BaseRecAdapter;
import com.xxrv.video.base.BaseRecViewHolder;
import com.xxrv.video.bean.circle.MusicInfo;
import com.xxrv.video.bean.circle.VideoIMTokenBean;
import com.xxrv.video.dialog.SelectMusicDialog;
import com.xxrv.video.presenter.VideoPresenter;
import com.xxrv.video.util.FileUtils;
import com.xxrv.video.util.ScreenUtil;
import com.xxrv.video.util.downloader.DownloadListener;
import com.xxrv.video.util.downloader.Downloader;
import com.xxrv.video.util.downloader.FailReason;
import com.yuyife.okgo.OkGoUtil;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * 选择音乐
 */
public class SelectMusicActivity extends BaseStatusActivity<VideoPresenter> implements View.OnClickListener {


    @BindView(R2.id.search_edit)
    EditText mEditText;
    @BindView(R2.id.tv_null_tip)
    TextView hint;
    @BindView(R2.id.rv_comm)
    RecyclerView mListView;
    LinearLayoutManager mLayoutManager;
    private Context mContext;
    private SelectMusicDialog.OnMusicItemClick mListener;
    private List<MusicInfo> mdatas;
    private List<MusicInfo> searchDatas;
    private CommAvatarAdapter mAdapter;
    private int mPagerIndex;
    private boolean isLoad;
    private int mCurrPlay = -1; // 当前选择的索引
    private MusicInfo mCurrMusic;
    private boolean more = true; // 是否可加载更多
    private Timer SEARCH_TIMER;
    private SearchTimerTask mSearchTask;
    private boolean isSearch; // 是否正在搜索
    private VideoIMTokenBean intokenBean;
    TextWatcher changeListener = new TextWatcher() {
        @Override
        public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {
        }

        @Override
        public void afterTextChanged(Editable s) {
            if (s.length() > 0) {
                mAdapter.setNewData(null);
                startSearchTimer(s.toString());
            } else {
                cancelSearchTimer();
                mAdapter.setNewData(mdatas);
                hint.setVisibility(View.GONE);
                more = true;
            }
        }
    };

    public static Intent getIntent(Context context) {
        return new Intent(context, SelectMusicActivity.class);
    }

    @Override
    protected int setLayout() {
        return R.layout.activity_select_music;
    }

    @Override
    protected void initView(Bundle savedInstanceState, TitleView titleView, Intent intent) {
        titleView.setTitle("选择音乐");
        mEditText.addTextChangedListener(changeListener);

        mLayoutManager = new LinearLayoutManager(mContext);
        mListView.setLayoutManager(mLayoutManager);
        mAdapter = new CommAvatarAdapter(mdatas);
        mListView.setAdapter(mAdapter);

        addListener();
    }

    @Override
    protected void loadData(Bundle savedInstanceState, Intent intent) {
        super.loadData(savedInstanceState, intent);
        upData();
    }

    @Override
    public void onClick(View v) {

    }

    private void playMusic(int position) {
        if (!isSearch && mCurrPlay == position) {
            return;
        }

        VoiceManager.instance().stop();
        List<MusicInfo> data = isSearch ? searchDatas : mdatas;

        if (mCurrPlay > -1 && data.size() > mCurrPlay) {
            data.get(mCurrPlay).state = 0;
            mAdapter.notifyItemChanged(mCurrPlay);
        }

        mCurrPlay = position;
        mCurrMusic = data.get(position);

        MusicInfo info = data.get(position);

        // 去下载路径找
        Downloader.getInstance().init(FileUtils.mFilesDir);
        File file = Downloader.getInstance().getFile(info.getPath());

        Log.e("xuan", "down : " + info.getPath());
        if (file.exists()) {
            data.get(mCurrPlay).state = 1;
            mAdapter.notifyItemChanged(mCurrPlay);

            VoiceManager.instance().play(file);
        } else {
            data.get(mCurrPlay).state = 2;
            mAdapter.notifyItemChanged(mCurrPlay);
            Downloader.getInstance().addDownload(info.getPath(), new DownloadListener() {
                @Override
                public void onStarted(String uri, View view) {
                    Log.e("xuan", "开始下载");
                }

                @Override
                public void onFailed(String uri, FailReason failReason, View view) {
                    Log.e("xuan", "onFailed: 下载失败" + mCurrMusic.getPath());
                    Toast.makeText(mContext, mContext.getString(R.string.tip_file_download_failed), Toast.LENGTH_SHORT).show();
                }

                @Override
                public void onComplete(String uri, String filePath, View view) {
                    Log.e("xuan", "comp: " + filePath);
                    if (Downloader.getInstance().getFile(mCurrMusic.getPath()).getAbsolutePath().equals(filePath)) {
                        mCurrMusic.setPath(filePath);
                        VoiceManager.instance().play(filePath);
                        data.get(mCurrPlay).state = 1;
                        mAdapter.notifyItemChanged(mCurrPlay);
                    }
                }

                @Override
                public void onCancelled(String uri, View view) {
                    Log.e("xuan", "取消下载");
                }
            });
        }
    }

    private void upData() {
        if (OkGoUtil.getToken() != null) {
            Map<String, Object> headMap = new LinkedHashMap<>();
            headMap.put("Authorization", OkGoUtil.getToken());
            mPresenter.postData(RvFrameConfig.HOST, 1, VideoApi.DISCOVERY_IM_LOGIN, VideoIMTokenBean.class, headMap, headMap, false);
        }
    }


    private void findMusicDataList(int index, String keyword) {
        Log.e("xuan", "findMusicDataList: 加载数据" + index);
        isLoad = true;

//        Map<String, String> params = new HashMap<>();
//        params.put("access_token", mToken);
//        params.put("pageIndex", String.valueOf(index));
//        params.put("pageSize", String.valueOf(20));
//        if (!TextUtils.isEmpty(keyword)) {
//            params.put("keyword", keyword);
//        }


//        HttpUtils.get().url(musicListUrl)
//                .params(params)
//                .build()
//                .execute(new ListCallback<MusicInfo>(MusicInfo.class) {
//                    @Override
//                    public void onResponse(ArrayResult<MusicInfo> result) {
//                        List<MusicInfo> data = result.getData();
//                        if (data.size() > 0) {
//
//                            for (int i = 0; i < data.size(); i++) {
//                                MusicInfo info = data.get(i);
//                                info.appendDown(mDown);
//                            }
//
//                            if (isSearch) {
//                                searchDatas.addAll(data);
//
//                                mAdapter.setNewData(searchDatas);
//                            } else {
//                                mdatas.addAll(data);
//                                mAdapter.setNewData(mdatas);
//                            }
//
//                            if (data.size() < 20) {
//                                more = false;
//                            }
//                            hint.setVisibility(View.GONE);
//                        } else {
//                            more = false;
//                            hint.setVisibility(View.VISIBLE);
//                        }
//                        isLoad = false;
//                    }
//
//                    @Override
//                    public void onError(Call call, Exception e) {
//                        isLoad = false;
//                    }
//                });
    }


    private void addListener() {
        mListView.addOnScrollListener(new RecyclerView.OnScrollListener() {
            int firstVisibleItem, visibleItemCount, totalItemCount;
            private int previousTotal = 0;

            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
            }

            @Override
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                if (!more) {
                    // 外界不让加载数据了
                    return;
                }
                visibleItemCount = recyclerView.getChildCount();
                totalItemCount = mLayoutManager.getItemCount();
                firstVisibleItem = mLayoutManager.findFirstVisibleItemPosition();

                if (isLoad) {
                    if (totalItemCount > previousTotal) {
                        isLoad = false;
                        previousTotal = totalItemCount;
                    }
                }
                if (!isLoad && (totalItemCount - visibleItemCount) <= firstVisibleItem) {
                    mPagerIndex++;
                    findMusicDataList(mPagerIndex, "");
                    isLoad = true;
                }
            }
        });
    }

    public void startSearchTimer(String key) {
        cancelSearchTimer();
        isSearch = true;
        searchDatas.clear();
        SEARCH_TIMER = new Timer();
        mSearchTask = new SearchTimerTask();
        mSearchTask.setKey(key);
        SEARCH_TIMER.schedule(mSearchTask, 400);
    }

    public void cancelSearchTimer() {
        isSearch = false;
        if (SEARCH_TIMER != null) {
            SEARCH_TIMER.cancel();
        }
        if (mSearchTask != null) {
            mSearchTask.cancel();
        }
    }

    @Override
    public void onShowResult(int requestType, BaseBean result) {
        switch (requestType) {
            case 1:
                intokenBean = (VideoIMTokenBean) result;
                setData(mPagerIndex, "");
                break;
            case 2:

                break;
        }
    }

    private void setData(int index, String keyword) {
        Map<String, Object> map = new LinkedHashMap<>();
        map.put("pageIndex", String.valueOf(index));
        map.put("access_token", intokenBean.getData());
        if (!TextUtils.isEmpty(keyword)) {
            map.put("keyword", keyword);
        }
        map.put("type", 4);
        mPresenter.postData(2, VideoApi.DISCOVERY_IM_MUSIC, BaseBean.class, map, index == 1 ? false : false);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // TODO: add setContentView(...) invocation
        ButterKnife.bind(this);
    }

    public interface OnMusicItemClick {
        void onItemClick(MusicInfo info);
    }

    class CommAvatarAdapter extends BaseRecAdapter<MusicInfo, CommAvatarHolder> {
        public CommAvatarAdapter(List<MusicInfo> list) {
            super(list);
        }

        @Override
        public void onHolder(CommAvatarHolder holder, MusicInfo bean, int position) {
            holder.tvName.setText(bean.getName());
            holder.tvContent.setText(bean.getNikeName());
            GlideApp.with(mContext).load(bean.getCover()).placeholder(R.drawable.defaultpic)
                    .error(R.drawable.image_download_fail_icon)
                    .into(holder.ivAvatar);

            if (bean.state == 0) {
                holder.mLlWrap.setVisibility(View.GONE);
                holder.ivState.setImageResource(R.drawable.ic_music_state0);
            } else if (bean.state == 1) {
                holder.mLlWrap.setVisibility(View.VISIBLE);
                holder.ivState.setImageResource(R.drawable.ic_music_state2);
            } else if (bean.state == 2) {
                holder.mLlWrap.setVisibility(View.GONE);
                holder.ivState.setImageResource(R.drawable.ic_music_state1);
            } else if (bean.state == 3) {
                holder.mLlWrap.setVisibility(View.VISIBLE);
                holder.ivState.setImageResource(R.drawable.ic_music_state0);
            }
        }

        @Override
        public CommAvatarHolder onCreateHolder() {
            return new CommAvatarHolder(getViewByRes(R.layout.item_trill_music));
        }
    }

    public class CommAvatarHolder extends BaseRecViewHolder {
        public TextView tvName;
        public TextView tvContent;
        public ImageView ivAvatar;
        public ImageView ivState;
        public LinearLayout mLlWrap;

        View.OnClickListener listener = v -> playMusic(getAdapterPosition());

        public CommAvatarHolder(View rootView) {
            super(rootView);

            tvName = rootView.findViewById(R.id.tv_name);
            tvContent = rootView.findViewById(R.id.tv_nike_name);
            ivAvatar = rootView.findViewById(R.id.iv_cover);
            mLlWrap = rootView.findViewById(R.id.ll_next);
            ivState = rootView.findViewById(R.id.iv_play_state);
            ivAvatar.setOnClickListener(listener);
            rootView.findViewById(R.id.rl_content).setOnClickListener(listener);

            mLlWrap.setOnClickListener(v -> {
                VoiceManager.instance().stop();
//                SelectMusicDialog.this.dismiss();
                if (mListener != null) {

                    List<MusicInfo> data = isSearch ? searchDatas : mdatas;
                    MusicInfo info = data.get(getAdapterPosition());
                    File file = Downloader.getInstance().getFile(info.getPath());
                    if (file.exists()) { // 动态设置一下
                        info.setPath(file.getAbsolutePath());
                    }
                    mListener.onItemClick(info);
                }
            });
        }
    }

    public class SearchTimerTask extends TimerTask {
        private String keyword;

        @Override
        public void run() {
            findMusicDataList(0, keyword);
        }

        public void setKey(String key) {
            this.keyword = key;
        }
    }
}
